@@ -3,7 +3,7 @@
  *
  * minimodem - software audio Bell-type or RTTY FSK modem
  *
- * Copyright (C) 2011 Kamal Mostafa <kamal@whence.com>
+ * Copyright (C) 2011-2012 Kamal Mostafa <kamal@whence.com>
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -40,6 +40,8 @@
 #include "fsk.h"
 #include "baudot.h"
 
+char *program_name = "";
+
 
 /*
  * ASCII 8-bit data framebits decoder/encoder (passthrough)
@@ -207,7 +209,7 @@ report_no_carrier( fsk_plan *fskp,
     fprintf(stderr, "nsamples_per_bit=%f\n", nsamples_per_bit);
 #endif
     float throughput_rate = nbits_total * sample_rate / (float)carrier_nsamples;
-    fprintf(stderr, "### NOCARRIER ndata=%u confidence=%.2f throughput=%.2f",
+    fprintf(stderr, "### NOCARRIER ndata=%u confidence=%.3f throughput=%.2f",
 	    nframes_decoded,
 	    confidence_total / nframes_decoded,
 	    throughput_rate);
@@ -223,12 +225,86 @@ report_no_carrier( fsk_plan *fskp,
     }
 }
 
+void
+generate_test_tones( simpleaudio *sa_out, unsigned int duration_sec )
+{
+    unsigned int sample_rate = simpleaudio_get_rate(sa_out);
+    unsigned int nframes = sample_rate / 10;
+    int i;
+    for ( i=0; i<(sample_rate/nframes*duration_sec); i++ ) {
+	simpleaudio_tone(sa_out, 1000, nframes/2);
+	simpleaudio_tone(sa_out, 1777, nframes/2);
+    }
+}
+
+static int
+benchmarks()
+{
+    fprintf(stdout, "minimodem %s benchmarks\n", VERSION);
+
+    int ret;
+    ret = system("sed -n -e '/^model name/{p;q}' -e '/^cpu model/{p;q}' /proc/cpuinfo");
+    ret = ret;	// don't care, hush compiler.
+
+    fflush(stdout);
+
+    unsigned int sample_rate = 48000;
+    sa_backend_t backend = SA_BACKEND_BENCHMARK;
+    // backend = SA_BACKEND_SYSDEFAULT;	// for test
+
+    simpleaudio *sa_out;
+
+
+    // enable the sine wave LUT
+    simpleaudio_tone_init(1024);
+
+    sa_out = simpleaudio_open_stream(backend, SA_STREAM_PLAYBACK,
+			SA_SAMPLE_FORMAT_S16, sample_rate, 1,
+			program_name, "generate-tones-lut1024-S16-mono");
+    if ( ! sa_out )
+	return 0;
+    generate_test_tones(sa_out, 10);
+    simpleaudio_close(sa_out);
+
+    sa_out = simpleaudio_open_stream(backend, SA_STREAM_PLAYBACK,
+			SA_SAMPLE_FORMAT_FLOAT, sample_rate, 1,
+			program_name, "generate-tones-lut1024-FLOAT-mono");
+    if ( ! sa_out )
+	return 0;
+    generate_test_tones(sa_out, 10);
+    simpleaudio_close(sa_out);
+
+
+    // disable the sine wave LUT
+    simpleaudio_tone_init(0);
+
+    sa_out = simpleaudio_open_stream(backend, SA_STREAM_PLAYBACK,
+			SA_SAMPLE_FORMAT_S16, sample_rate, 1,
+			program_name, "generate-tones-nolut-S16-mono");
+    if ( ! sa_out )
+	return 0;
+    generate_test_tones(sa_out, 10);
+    simpleaudio_close(sa_out);
+
+    sa_out = simpleaudio_open_stream(backend, SA_STREAM_PLAYBACK,
+			SA_SAMPLE_FORMAT_FLOAT, sample_rate, 1,
+			program_name, "generate-tones-nolut-FLOAT-mono");
+    if ( ! sa_out )
+	return 0;
+    generate_test_tones(sa_out, 10);
+    simpleaudio_close(sa_out);
+
+
+    return 1;
+}
+
+
 void
 version()
 {
     printf(
     "minimodem %s\n"
-    "Copyright (C) 2011 Kamal Mostafa <kamal@whence.com>\n"
+    "Copyright (C) 2011-2012 Kamal Mostafa <kamal@whence.com>\n"
     "License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>.\n"
     "This is free software: you are free to change and redistribute it.\n"
     "There is NO WARRANTY, to the extent permitted by law.\n\n"
@@ -245,7 +321,8 @@ usage()
     "		    -r, --rx, --receive,  --read     (default)\n"
     "		[options]\n"
     "		    -a, --auto-carrier\n"
-    "		    -c, --confidence {threshold}\n"
+    "		    -c, --confidence {min-snr-threshold}\n"
+    "		    -l, --limit {max-snr-search-limit}\n"
     "		    -8, --ascii		ASCII  8-N-1\n"
     "		    -5, --baudot	Baudot 5-N-1\n"
     "		    -f, --file {filename.flac}\n"
@@ -254,13 +331,17 @@ usage()
     "		    -S, --space {space_freq}\n"
     "		    -T, --txstopbits {m.n}\n"
     "		    -q, --quiet\n"
+    "		    -R, --samplerate {rate}\n"
     "		    -V, --version\n"
+    "		    -A, --alsa\n"
+    "		    --lut={tx_sin_table_len}\n"
+    "		    --float-samples\n"
+    "		    --benchmarks\n"
     "		{baudmode}\n"
-    "		    1200 : Bell202  1200 bps --ascii\n"
-    "		     300 : Bell103   300 bps --ascii\n"
-    "		  N>=100 : Bellxxx     N bps --ascii\n"
-    "		   N<100 : RTTY        N bps --baudot\n"
-    "		    rtty : RTTY    45.45 bps --baudot\n"
+    "		    1200       Bell202  1200 bps --ascii\n"
+    "		     300       Bell103   300 bps --ascii\n"
+    "	    any_number_N       Bell103     N bps --ascii\n"
+    "		    rtty       RTTY    45.45 bps --baudot\n"
     );
     exit(1);
 }
@@ -278,10 +359,37 @@ main( int argc, char*argv[] )
     unsigned int bfsk_n_data_bits = 0;
     int autodetect_shift;
     char *filename = NULL;
-    char *program_name;
 
     float	carrier_autodetect_threshold = 0.0;
-    float	bfsk_confidence_threshold = 0.6;
+
+    // fsk_confidence_threshold : signal-to-noise squelch control
+    //
+    // The minimum SNR confidence level seen as "a signal".
+    float fsk_confidence_threshold = 2.0;
+
+    // fsk_confidence_search_limit : performance vs. quality
+    //
+    // If we find a frame with SNR confidence > confidence_search_limit,
+    // quit searching for a better frame.  confidence_search_limit has a
+    // dramatic effect on peformance (high value yields low performance, but
+    // higher decode quality, for noisy or hard-to-discern signals (Bell 103).
+    float fsk_confidence_search_limit = 2.3f;
+    // float fsk_confidence_search_limit = INFINITY;  /* for test */
+
+    sa_backend_t sa_backend = SA_BACKEND_SYSDEFAULT;
+    sa_format_t sample_format = SA_SAMPLE_FORMAT_S16;
+    unsigned int sample_rate = 48000;
+    unsigned int nchannels = 1; // FIXME: only works with one channel
+
+    unsigned int tx_sin_table_len = 4096;
+
+    /* validate the default system audio mechanism */
+#if !(USE_PULSEAUDIO || USE_ALSA)
+# define _MINIMODEM_NO_SYSTEM_AUDIO
+# if !USE_SNDFILE
+#  error At least one of {USE_PULSEAUDIO,USE_ALSA,USE_SNDFILE} must be enabled!
+# endif
+#endif
 
     program_name = strrchr(argv[0], '/');
     if ( program_name )
@@ -292,6 +400,13 @@ main( int argc, char*argv[] )
     int c;
     int option_index;
     
+    enum {
+	MINIMODEM_OPT_UNUSED=256,	// placeholder
+	MINIMODEM_OPT_LUT,
+	MINIMODEM_OPT_FLOAT_SAMPLES,
+	MINIMODEM_OPT_BENCHMARKS,
+    };
+
     while ( 1 ) {
 	static struct option long_options[] = {
 	    { "version",	0, 0, 'V' },
@@ -302,6 +417,7 @@ main( int argc, char*argv[] )
 	    { "receive",	0, 0, 'r' },
 	    { "read",		0, 0, 'r' },
 	    { "confidence",	1, 0, 'c' },
+	    { "limit",		1, 0, 'l' },
 	    { "auto-carrier",	0, 0, 'a' },
 	    { "ascii",		0, 0, '8' },
 	    { "baudot",		0, 0, '5' },
@@ -311,9 +427,14 @@ main( int argc, char*argv[] )
 	    { "space",		1, 0, 'S' },
 	    { "txstopbits",	1, 0, 'T' },
 	    { "quiet",		0, 0, 'q' },
+	    { "alsa",		0, 0, 'A' },
+	    { "samplerate",	1, 0, 'R' },
+	    { "lut",		1, 0, MINIMODEM_OPT_LUT },
+	    { "float-samples",	0, 0, MINIMODEM_OPT_FLOAT_SAMPLES },
+	    { "benchmarks",	0, 0, MINIMODEM_OPT_BENCHMARKS },
 	    { 0 }
 	};
-	c = getopt_long(argc, argv, "Vtrc:a85f:b:M:S:T:q",
+	c = getopt_long(argc, argv, "Vtrc:a85f:b:M:S:T:qAR:",
 		long_options, &option_index);
 	if ( c == -1 )
 	    break;
@@ -332,7 +453,10 @@ main( int argc, char*argv[] )
 			TX_mode = 0;
 			break;
 	    case 'c':
-			bfsk_confidence_threshold = atof(optarg);
+			fsk_confidence_threshold = atof(optarg);
+			break;
+	    case 'l':
+			fsk_confidence_search_limit = atof(optarg);
 			break;
 	    case 'a':
 			carrier_autodetect_threshold = 0.001;
@@ -365,6 +489,28 @@ main( int argc, char*argv[] )
 	    case 'q':
 			quiet_mode = 1;
 			break;
+	    case 'R':
+			sample_rate = atoi(optarg);
+			assert( sample_rate > 0 );
+			break;
+	    case 'A':
+#if USE_ALSA
+			sa_backend = SA_BACKEND_ALSA;
+#else
+			fprintf(stderr, "E: This build of minimodem was configured without alsa support.\n");
+			exit(1);
+#endif
+			break;
+	    case MINIMODEM_OPT_LUT:
+			tx_sin_table_len = atoi(optarg);
+			break;
+	    case MINIMODEM_OPT_FLOAT_SAMPLES:
+			sample_format = SA_SAMPLE_FORMAT_FLOAT;
+			break;
+	    case MINIMODEM_OPT_BENCHMARKS:
+			benchmarks();
+			exit(0);
+			break;
 	    default:
 			usage();
 	}
@@ -372,6 +518,22 @@ main( int argc, char*argv[] )
     if ( TX_mode == -1 )
 	TX_mode = 0;
 
+    /* The receive code requires floating point samples to feed to the FFT */
+    if ( TX_mode == 0 )
+	sample_format = SA_SAMPLE_FORMAT_FLOAT;
+
+    if ( filename ) {
+#if !USE_SNDFILE
+	fprintf(stderr, "E: This build of minimodem was configured without sndfile,\nE:   so the --file flag is not supported.\n");
+	exit(1);
+#endif
+    } else {
+#ifdef _MINIMODEM_NO_SYSTEM_AUDIO
+	fprintf(stderr, "E: this build of minimodem was configured without system audio support,\nE:   so only the --file mode is supported.\n");
+	exit(1);
+#endif
+    }
+
 #if 0
     if (optind < argc) {
 	printf("non-option ARGV-elements: ");
@@ -463,25 +625,34 @@ main( int argc, char*argv[] )
     if ( band_width > bfsk_data_rate )
 	band_width = bfsk_data_rate;
 
+    // sanitize confidence search limit
+    if ( fsk_confidence_search_limit < fsk_confidence_threshold )
+	fsk_confidence_search_limit = fsk_confidence_threshold;
+
+    char *stream_name = NULL;;
+
+    if ( filename ) {
+	sa_backend = SA_BACKEND_FILE;
+	stream_name = filename;
+    }
 
     /*
      * Handle transmit mode
      */
     if ( TX_mode ) {
 
-	simpleaudio *sa_out = NULL;
-	int tx_interactive = 1;
+	simpleaudio_tone_init(tx_sin_table_len);
 
-	if ( filename ) {
-	    tx_interactive = 0;
-	    sa_out = simpleaudio_open_stream_sndfile(SA_STREAM_PLAYBACK,
-					filename);
-	    if ( ! sa_out )
-		return 1;
+	int tx_interactive = 0;
+	if ( ! stream_name ) {
+	    tx_interactive = 1;
+	    stream_name = "output audio";
 	}
-	if ( ! sa_out )
-	    sa_out = simpleaudio_open_stream_pulseaudio(SA_STREAM_PLAYBACK,
-					program_name, "output audio");
+
+	simpleaudio *sa_out;
+	sa_out = simpleaudio_open_stream(sa_backend, SA_STREAM_PLAYBACK,
+					sample_format, sample_rate, nchannels,
+					program_name, stream_name);
 	if ( ! sa_out )
 	    return 1;
 
@@ -492,6 +663,9 @@ main( int argc, char*argv[] )
 				bfsk_txstopbits,
 				bfsk_framebits_encode
 				);
+
+	simpleaudio_close(sa_out);
+
 	return 0;
     }
 
@@ -499,23 +673,17 @@ main( int argc, char*argv[] )
     /*
      * Open the input audio stream
      */
-    simpleaudio *sa = NULL;
-    if ( filename ) {
-	sa = simpleaudio_open_stream_sndfile(SA_STREAM_RECORD, filename);
-	if ( ! sa )
-	    return 1;
-    }
+
+    if ( ! stream_name )
+	stream_name = "input audio";
+
+    simpleaudio *sa;
+    sa = simpleaudio_open_stream(sa_backend, SA_STREAM_RECORD,
+				sample_format, sample_rate, nchannels,
+				program_name, stream_name);
     if ( ! sa )
-	sa = simpleaudio_open_stream_pulseaudio(SA_STREAM_RECORD,
-				    program_name, "input audio");
-    if ( !sa )
         return 1;
 
-    unsigned int sample_rate = simpleaudio_get_rate(sa);
-    unsigned int nchannels = simpleaudio_get_channels(sa);
-
-    assert( nchannels == 1 );
-
 
     /*
      * Prepare the input sample chunk rate
@@ -543,12 +711,13 @@ main( int argc, char*argv[] )
 // FIXME I should be able to reduce this to * 9 for 5-bit data, but
 // it SOMETIMES crashes -- probably due to non-integer nsamples_per_bit
 // FIXME by passing it down into the fsk code?
-    size_t	samplebuf_size = ceilf(nsamples_per_bit) * 12;
+    // FIXME EXPLAIN +1 goes with extra bit when scanning
+    size_t	samplebuf_size = ceilf(nsamples_per_bit) * (12+1);
     float	*samplebuf = malloc(samplebuf_size * sizeof(float));
     float	*samples_readptr = samplebuf;
     size_t	read_nsamples = samplebuf_size;
     size_t	samples_nvalid = 0;
-    debug_log("samplebuf_size=%lu\n", samplebuf_size);
+    debug_log("samplebuf_size=%zu\n", samplebuf_size);
 
     /*
      * Run the main loop
@@ -562,16 +731,31 @@ main( int argc, char*argv[] )
     size_t		carrier_nsamples = 0;
 
     unsigned int	noconfidence = 0;
-    size_t		noconfidence_nsamples = 0;
     unsigned int	advance = 0;
 
+    // Fraction of nsamples_per_bit that we will "overscan"; range (0.0 .. 1.0)
+    float fsk_frame_overscan = 0.5;
+    //   should be != 0.0 (only the nyquist edge cases actually require this?)
+    // for handling of slightly faster-than-us rates:
+    //   should be >> 0.0 to allow us to lag back for faster-than-us rates
+    //   should be << 1.0 or we may lag backwards over whole bits
+    // for optimal analysis:
+    //   should be >= 0.5 (half a bit width) or we may not find the optimal bit
+    //   should be <  1.0 (a full bit width) or we may skip over whole bits
+    assert( fsk_frame_overscan >= 0.0 && fsk_frame_overscan < 1.0 );
+
+    // ensure that we overscan at least a single sample
+    unsigned int nsamples_overscan
+			= nsamples_per_bit * fsk_frame_overscan + 0.5;
+    if ( fsk_frame_overscan > 0.0 && nsamples_overscan == 0 )
+	nsamples_overscan = 1;
+    debug_log("fsk_frame_overscan=%f nsamples_overscan=%u\n",
+	    fsk_frame_overscan, nsamples_overscan);
+
     while ( 1 ) {
 
 	debug_log("advance=%u\n", advance);
 
-	if ( carrier && nframes_decoded > 0 )
-	    carrier_nsamples += advance;
-	
 	/* Shift the samples in samplebuf by 'advance' samples */
 	assert( advance <= samplebuf_size );
 	if ( advance == samplebuf_size ) {
@@ -595,7 +779,7 @@ main( int argc, char*argv[] )
 	assert ( samples_nvalid + read_nsamples <= samplebuf_size );
 	ssize_t r;
 	r = simpleaudio_read(sa, samples_readptr, read_nsamples);
-	debug_log("simpleaudio_read(samplebuf+%ld, n=%lu) returns %ld\n",
+	debug_log("simpleaudio_read(samplebuf+%zd, n=%zu) returns %zd\n",
 		samples_readptr - samplebuf, read_nsamples, r);
 	if ( r < 0 ) {
 	    fprintf(stderr, "simpleaudio_read: error\n");
@@ -660,7 +844,12 @@ main( int argc, char*argv[] )
 	if ( samples_nvalid < frame_nsamples )
 	    break;
 
-	unsigned int try_max_nsamples = nsamples_per_bit;
+	// try_max_nsamples = nsamples_per_bit + nsamples_overscan;
+	// serves two purposes
+	// 1. avoids finding a non-optimal first frame
+	// 2. allows us to track slightly slow signals
+	unsigned int try_max_nsamples = nsamples_per_bit + nsamples_overscan;
+
 #define FSK_ANALYZE_NSTEPS		10	/* accuracy vs. performance */
 		// Note: FSK_ANALYZE_NSTEPS has subtle effects on the
 		// "rate perfect" calculation.  oh well.
@@ -674,9 +863,25 @@ main( int argc, char*argv[] )
 	 * prev_stop bit begins (since the "frame" includes the prev_stop). */
 	unsigned int frame_start_sample = 0;
 
+	// If we don't have carrier, then set this try_confidence_search_limit
+	// to infinity (search for best possible frame) so to get the decoder
+	// into phase with the signal, so the next try_first_sample will match
+	// up with where the next frame should be.
+	unsigned int try_first_sample;
+	float try_confidence_search_limit;
+	if ( carrier ) {
+	    try_first_sample = nsamples_overscan;
+	    try_confidence_search_limit = fsk_confidence_search_limit;
+	} else {
+	    try_first_sample = 0;
+	    try_confidence_search_limit = INFINITY;
+	}
+
 	confidence = fsk_find_frame(fskp, samplebuf, frame_nsamples,
+			try_first_sample,
 			try_max_nsamples,
 			try_step_nsamples,
+			try_confidence_search_limit,
 			&bits,
 			&frame_start_sample
 			);
@@ -689,17 +894,13 @@ main( int argc, char*argv[] )
 
 #define FSK_MAX_NOCONFIDENCE_BITS	20
 
-#define FSK_SCAN_LAG			0.2
+	if ( confidence <= fsk_confidence_threshold ) {
 
-	if ( confidence <= bfsk_confidence_threshold ) {
 	    // FIXME: explain
 	    if ( ++noconfidence > FSK_MAX_NOCONFIDENCE_BITS )
 	    {
 		carrier_band = -1;
 		if ( carrier ) {
-		    carrier_nsamples -= noconfidence_nsamples;
-		    if ( nframes_decoded > 0 )
-			carrier_nsamples += nsamples_per_bit * FSK_SCAN_LAG;
 		    if ( !quiet_mode )
 			report_no_carrier(fskp, sample_rate, bfsk_data_rate,
 			    nsamples_per_bit, nframes_decoded,
@@ -715,11 +916,20 @@ main( int argc, char*argv[] )
 	     * next time around the loop we continue searching from where
 	     * we left off this time.		*/
 	    advance = try_max_nsamples;
-	    noconfidence_nsamples += advance;
+	    debug_log("@ NOCONFIDENCE=%u advance=%u\n", noconfidence, advance);
 	    continue;
 	}
 
-	if ( !carrier ) {
+	// Add a frame's worth of samples to the sample count
+	carrier_nsamples += nsamples_per_bit * (fskp->n_data_bits + 2);
+
+	if ( carrier ) {
+
+	    // If we already had carrier, adjust sample count +start -overscan
+	    carrier_nsamples += frame_start_sample;
+	    carrier_nsamples -= nsamples_overscan;
+
+	} else {
 	    if ( !quiet_mode ) {
 		if ( bfsk_data_rate >= 100 )
 		    fprintf(stderr, "### CARRIER %u @ %.1f Hz ###\n",
@@ -731,8 +941,6 @@ main( int argc, char*argv[] )
 			    fskp->b_mark * fskp->band_width);
 	    }
 	    carrier = 1;
-	    /* back up carrier_nsamples to account for the imminent advance */
-	    noconfidence_nsamples = frame_start_sample;
 	    bfsk_framebits_decode(0, 0, 0);	/* reset the frame processor */
 	}
 
@@ -748,10 +956,11 @@ main( int argc, char*argv[] )
 	 * advance = 1 prev_stop + 1 start + N data bits == n_data_bits+2
 	 *
 	 * but actually advance just a bit less than that to allow
-	 * for clock skew, hence FSK_SCAN_LAG.
+	 * for tracking slightly fast signals, hence - nsamples_overscan.
 	 */
-	advance = frame_start_sample +
-	    nsamples_per_bit * (float)(fskp->n_data_bits + 2 - FSK_SCAN_LAG);
+	advance = frame_start_sample
+		+ nsamples_per_bit * (float)(fskp->n_data_bits + 2)
+		- nsamples_overscan;
 
 	debug_log("@ nsamples_per_bit=%.3f n_data_bits=%u "
 			" frame_start=%u advance=%u\n",
@@ -787,9 +996,6 @@ main( int argc, char*argv[] )
     } /* end of the main loop */
 
     if ( carrier ) {
-	carrier_nsamples -= noconfidence_nsamples;
-	if ( nframes_decoded > 0 )
-	    carrier_nsamples += nsamples_per_bit * FSK_SCAN_LAG;
 	if ( !quiet_mode )
 	    report_no_carrier(fskp, sample_rate, bfsk_data_rate,
 		nsamples_per_bit, nframes_decoded,
