@@ -1,7 +1,7 @@
 /*
  * simpleaudio-pulse.c
  *
- * Copyright (C) 2011 Kamal Mostafa <kamal@whence.com>
+ * Copyright (C) 2011-2012 Kamal Mostafa <kamal@whence.com>
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,6 +17,11 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#if USE_PULSEAUDIO
 
 #include <stdio.h>
 #include <malloc.h>
@@ -36,7 +41,7 @@
 
 
 static ssize_t
-sa_pulse_read( simpleaudio *sa, float *buf, size_t nframes )
+sa_pulse_read( simpleaudio *sa, void *buf, size_t nframes )
 {
     int error;
     pa_simple *s = (pa_simple *)sa->backend_handle;
@@ -52,7 +57,7 @@ sa_pulse_read( simpleaudio *sa, float *buf, size_t nframes )
 
 
 static ssize_t
-sa_pulse_write( simpleaudio *sa, float *buf, size_t nframes )
+sa_pulse_write( simpleaudio *sa, void *buf, size_t nframes )
 {
     int error;
     pa_simple *s = (pa_simple *)sa->backend_handle;
@@ -74,17 +79,12 @@ sa_pulse_close( simpleaudio *sa )
     pa_simple_free(sa->backend_handle);
 }
 
-
-static const struct simpleaudio_backend simpleaudio_backend_pulse = {
-    sa_pulse_read,
-    sa_pulse_write,
-    sa_pulse_close,
-};
-
-simpleaudio *
-simpleaudio_open_stream_pulseaudio(
-		// unsigned int rate, unsigned int channels,
-		int sa_stream_direction,
+static int
+sa_pulse_open_stream(
+		simpleaudio *sa,
+		sa_direction_t sa_stream_direction,
+		sa_format_t sa_format,
+		unsigned int rate, unsigned int channels,
 		char *app_name, char *stream_name )
 {
     int error;
@@ -92,13 +92,24 @@ simpleaudio_open_stream_pulseaudio(
     // FIXME - use source for something
     // just take the default pulseaudio source for now
 
+    pa_sample_format_t pa_format;
+
+    switch ( sa->format ) {
+	case SA_SAMPLE_FORMAT_FLOAT:
+		pa_format = PA_SAMPLE_FLOAT32;
+		break;
+	case SA_SAMPLE_FORMAT_S16:
+		pa_format = PA_SAMPLE_S16LE;	// FIXME: handle S16BE
+		break;
+	default:
+		assert(0);
+    }
+
     /* The sample type to use */
     pa_sample_spec ss = {
-        .format = PA_SAMPLE_FLOAT32,
-	// .rate = rate,
-	.rate = 48000,
-	// .channels = channels,
-	.channels = 1,
+        .format = pa_format,
+	.rate = rate,
+	.channels = channels,
     };
 
     pa_buffer_attr attr = {
@@ -122,23 +133,25 @@ simpleaudio_open_stream_pulseaudio(
 	    &ss, NULL, &attr, &error);
     if ( !s ) {
         fprintf(stderr, "E: Cannot create PulseAudio stream: %s\n ", pa_strerror(error));
-        return NULL;
+        return 0;
     }
 
-    simpleaudio *sa = malloc(sizeof(simpleaudio));
-    if ( !sa ) {
-	perror("malloc");
-	pa_simple_free(s);
-        return NULL;
-    }
+    /* good or bad to override these? */
     sa->rate = ss.rate;
     sa->channels = ss.channels;
-    sa->backend = &simpleaudio_backend_pulse;
+
     sa->backend_handle = s;
     sa->backend_framesize = pa_frame_size(&ss);
 
-    assert( sa->backend_framesize == ss.channels * sizeof(float) );
-
-    return sa;
+    return 1;
 }
 
+
+const struct simpleaudio_backend simpleaudio_backend_pulseaudio = {
+    sa_pulse_open_stream,
+    sa_pulse_read,
+    sa_pulse_write,
+    sa_pulse_close,
+};
+
+#endif /* USE_PULSEAUDIO */
