@@ -1,7 +1,7 @@
 /*
  * simpleaudio-sndfile.c
  *
- * Copyright (C) 2011 Kamal Mostafa <kamal@whence.com>
+ * Copyright (C) 2011-2012 Kamal Mostafa <kamal@whence.com>
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,6 +17,11 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#if USE_SNDFILE
 
 #include <stdio.h>
 #include <malloc.h>
@@ -35,28 +40,50 @@
 
 
 static ssize_t
-sa_sndfile_read( simpleaudio *sa, float *buf, size_t nframes )
+sa_sndfile_read( simpleaudio *sa, void *buf, size_t nframes )
 {
     SNDFILE *s = (SNDFILE *)sa->backend_handle;
     int n;
-    if ((n = sf_readf_float(s, buf, nframes)) < 0) {
-	fprintf(stderr, "sf_read_float: ");
+    switch ( sa->format ) {
+	case SA_SAMPLE_FORMAT_FLOAT:
+		n = sf_readf_float(s, buf, nframes);
+		break;
+	case SA_SAMPLE_FORMAT_S16:
+		n = sf_readf_short(s, buf, nframes);
+		break;
+	default:
+		assert(0);
+		break;
+    }
+    if ( n < 0 ) {
+	fprintf(stderr, "sf_read: ");
 	sf_perror(s);
 	return -1;
     }
-//fprintf(stderr, "sf_readf_float: nframes=%ld n=%d\n", nframes, n);
+    // fprintf(stderr, "sf_read: nframes=%ld n=%d\n", nframes, n);
     return n;
 }
 
 
 static ssize_t
-sa_sndfile_write( simpleaudio *sa, float *buf, size_t nframes )
+sa_sndfile_write( simpleaudio *sa, void *buf, size_t nframes )
 {
-//fprintf(stderr, "sf_writef_float: nframes=%ld\n", nframes);
+    // fprintf(stderr, "sf_write: nframes=%ld\n", nframes);
     SNDFILE *s = (SNDFILE *)sa->backend_handle;
     int n;
-    if ((n = sf_writef_float(s, buf, nframes)) < 0) {
-	fprintf(stderr, "sf_read_float: ");
+    switch ( sa->format ) {
+	case SA_SAMPLE_FORMAT_FLOAT:
+		n = sf_writef_float(s, buf, nframes);
+		break;
+	case SA_SAMPLE_FORMAT_S16:
+		n = sf_writef_short(s, buf, nframes);
+		break;
+	default:
+		assert(0);
+		break;
+    }
+    if ( n < 0 ) {
+	fprintf(stderr, "sf_write: ");
 	sf_perror(s);
 	return -1;
     }
@@ -71,13 +98,6 @@ sa_sndfile_close( simpleaudio *sa )
 }
 
 
-static const struct simpleaudio_backend simpleaudio_backend_sndfile = {
-    sa_sndfile_read,
-    sa_sndfile_write,
-    sa_sndfile_close,
-};
-
-
 /* (Why) doesn't libsndfile provide an API for this?... */
 static const struct sndfile_format {
     unsigned int major_format;
@@ -127,20 +147,37 @@ sndfile_format_from_path( const char *path )
     return SF_FORMAT_WAV;
 }
 
-simpleaudio *
-simpleaudio_open_stream_sndfile(
-		int sa_stream_direction,
-		char *path )
+static int
+sa_sndfile_open_stream(
+		simpleaudio *sa,
+		sa_direction_t sa_stream_direction,
+		sa_format_t sa_format,
+		unsigned int rate, unsigned int channels,
+		char *app_name, char *stream_name )
 {
+    const char *path = stream_name;
+
+    int sf_format;
+    switch ( sa->format ) {
+	case SA_SAMPLE_FORMAT_FLOAT:
+		sf_format = SF_FORMAT_FLOAT;
+		break;
+	case SA_SAMPLE_FORMAT_S16:
+		sf_format = SF_FORMAT_PCM_16;
+		break;
+	default:
+		assert(0);
+    }
+
     /* setting for SA_STREAM_PLAYBACK (file write) */
     SF_INFO sfinfo = {
-	.format = 0,
-	.samplerate = 48000,
-	.channels = 1,
+	.format = sf_format,
+	.samplerate = rate,
+	.channels = channels,
     };
 
     if ( sa_stream_direction == SA_STREAM_PLAYBACK )
-	sfinfo.format = sndfile_format_from_path(path) | SF_FORMAT_PCM_16;
+	sfinfo.format = sndfile_format_from_path(path) | sf_format;
 
     /* Create the recording stream */
     SNDFILE *s;
@@ -150,21 +187,25 @@ simpleaudio_open_stream_sndfile(
     if ( !s ) {
 	fprintf(stderr, "%s: ", path);
 	sf_perror(s);
-        return NULL;
+        return 0;
     }
 
-    simpleaudio *sa = malloc(sizeof(simpleaudio));
-    if ( !sa ) {
-	perror("malloc");
-	sf_close(s);
-        return NULL;
-    }
+    /* good or bad to override these? */
     sa->rate = sfinfo.samplerate;
     sa->channels = sfinfo.channels;
-    sa->backend = &simpleaudio_backend_sndfile;
-    sa->backend_handle = s;
-    sa->backend_framesize = sa->channels * sizeof(float);
 
-    return sa;
+    sa->backend_handle = s;
+    sa->backend_framesize = sa->channels * sa->samplesize; 
+
+    return 1;
 }
 
+
+const struct simpleaudio_backend simpleaudio_backend_sndfile = {
+    sa_sndfile_open_stream,
+    sa_sndfile_read,
+    sa_sndfile_write,
+    sa_sndfile_close,
+};
+
+#endif /* USE_SNDFILE */
